/* 
   gen_audio_core.c (C) Heikki Lindholm 2003

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

/*
 * - Simple generic core to choose between the available real cores
 * - in principle much like v4lgen_core
 */

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#include <pthread.h>
#include <signal.h>

#include "audio_core_common.h"

void gen_audio_core_usage()
{
	MPRINT("Audio capture core usage:\n");
#ifdef HAVE_ALSA
	MPRINT("\t-ua <core name> use audio core       [ALSA]\n");
	alsa_core.aud_core_usage();
#endif
	oss_core.aud_core_usage();
}

struct aud_core_t * gen_audio_core_init(char ** args, ntime_t ftime)
{
        struct aud_core_t * act = NULL;
	int use_core = -1; // 0 OSS, 1 ALSA
	NV_ARGS;
        
        NV_SCAN_ARGS {
                if(strcmp(NV_THIS_ARG, "-ua") == 0) {
                        if(!NV_NEXT_ARG) {
                                EPRINT("Missing argument to -ua.\n");
                                goto usage;
                        }
                        if (strcasecmp(NV_THIS_ARG, "ALSA") == 0)
				use_core = 1;
			else if (strcasecmp(NV_THIS_ARG, "OSS") == 0)
				use_core = 0;
			else if (strcasecmp(NV_THIS_ARG, "help") == 0) { 
#ifdef HAVE_ALSA
				MPRINT("Available audio core options: ALSA OSS\n");
#else
				MPRINT("Available audio core options: OSS\n");
#endif
				return NULL;
			}
                }
        }

#ifdef HAVE_ALSA
	if(use_core != 0) { /* Not OSS override */
		act = alsa_core.aud_core_init(args, ftime);
		if(!act) {
			IPRINT("ALSA core init failed - fall back to OSS\n");
		}
	}
#endif	
	if(!act) {
		act = oss_core.aud_core_init(args, ftime);
	}
	return act;

usage:
        EPRINT("Error parsing options.  Please run without any command line options for\n");
        EPRINT("detailed usage information.\n");
        return NULL;
}

struct aud_core_t gen_audio_core = {
	aud_core_init: gen_audio_core_init, 
	aud_core_close: NULL,
	aud_core_tailtime: NULL,
	aud_core_droptail: NULL,
	aud_core_read: NULL,
	aud_core_bps: NULL,
	aud_core_overrides: NULL,
	aud_core_usage: gen_audio_core_usage,
	priv: NULL
};
